@@ -232,12 +232,14 @@ class Context:
         feedback versions if provided are used instead of the originals
         when generating a feedback document as opposed to a blank rubric.
 
-        You must also supply a context_builder function, to be run when
-        this context is required. If depends is supplied, it should be a
-        list of Context objects, and before this context is established,
-        each of those contexts will be created and merged in turn, to
-        establish the prev_context argument to the context_builder
-        function.
+        You must also supply a builder function, to be run when this
+        context is required. If depends is supplied, it should be a list
+        of Context objects, and before this context is established, each
+        of those contexts will be created and merged in turn, to
+        establish the prev_context argument to the builder function. The
+        builder function must accept one argument (a context dictionary)
+        and must return a dictionary of any new or modified keys that it
+        wants to update in that context dictionary.
 
         You may supply a failure_explanation, which can be either a
         string, or a function that will be given the context in which the
@@ -990,13 +992,17 @@ class FileContext(AutoContext):
                 )
                 # Even if we're grading a custom-named file, the solution
                 # won't have a custom name...
-                ref_target = os.path.join(soln_root, default_filename)
+                ref_target = os.path.abspath(
+                    os.path.join(soln_root, default_filename)
+                )
                 target_filename = default_filename
             else:
                 # TODO: How can we properly handle ref targets for
                 # custom-named files here?
                 full_target = os.path.join(submission_root, file_target)
-                ref_target = os.path.join(soln_root, file_target)
+                ref_target = os.path.abspath(
+                    os.path.join(soln_root, file_target)
+                )
                 _, target_filename = os.path.split(full_target)
             if not os.path.exists(submission_root):
                 # If the submission root directory is missing...
@@ -1185,7 +1191,8 @@ class SolnCodeContext(AutoContext):
     `FileContext`), and establishes a "ref_source" slot which contains
     the raw text of the equivalent file from the solution code, along
     with a "ref_scope" slot which contains the parsed AST from the
-    solution code.
+    solution code. Also establishes "ref_original_source" which may be
+    different from "ref_source" when a prep function is used.
 
     "task_info" and "submission_root" slots are also required, but those
     should always be present.
@@ -1195,11 +1202,13 @@ class SolnCodeContext(AutoContext):
     `ContextCreationError` will be generated naming the relevant error as
     its cause.
     """
-    def __init__(self, depends=None, hidden=False):
+    def __init__(self, depends=None, hidden=False, prep=None):
         """
         Dependencies are optional; if not specified `auto` will be used
         to fill them in. `hidden` may be provided; by default this
-        context is not hidden.
+        context is not hidden. A `prep` function may be supplied, which
+        will be given the source code and its return value will be used
+        in place of the original source code.
         """
         # First, create our context builder
         def establish_context(prev_context):
@@ -1232,8 +1241,13 @@ class SolnCodeContext(AutoContext):
             with open(soln_equivalent, 'r', encoding="utf-8") as fin:
                 contents = fin.read()
 
+            if prep:
+                source = prep(contents)
+            else:
+                source = contents
+
             try:
-                node = mast.parse(contents, filename=ref_filename)
+                node = mast.parse(source, filename=ref_filename)
             except Exception as e:
                 raise context_utils.ContextCreationError(
                     self,
@@ -1242,7 +1256,8 @@ class SolnCodeContext(AutoContext):
                 )
 
             return {
-                "ref_source": contents,
+                "ref_original_source": contents,
+                "ref_source": source,
                 "ref_scope": node,
                 "ref_top_scope": node
             }
@@ -1268,7 +1283,12 @@ class SolnCodeContext(AutoContext):
 
         # Finally, register ourselves as an auto provider for the slots
         # that we generate:
-        self.register("ref_source", "ref_scope", "ref_top_scope")
+        self.register(
+            "ref_original_source",
+            "ref_source",
+            "ref_scope",
+            "ref_top_scope"
+        )
 
 
 # Register a factory for a default CodeContext as an on-demand option
@@ -1398,9 +1418,11 @@ class ModuleContext(AutoContext):
 
             # Set up phony stdin, so that stray inputs won't immediately
             # crash the program (if their results are used in a delicate
-            # manner, they still will of course).
+            # manner, they still will of course, but we supply '1' for
+            # each input, which will survive conversion to an int or
+            # float).
             old_stdin = sys.stdin
-            sys.stdin = context_utils.MashEnter()
+            sys.stdin = context_utils.AllOnes()
             # Note: we don't care about echoing inputs to stdout here...
 
             # Set up phony stdout and stderr
@@ -1427,6 +1449,8 @@ class ModuleContext(AutoContext):
                     established_expectations = optimism.deepish_copy(
                         optimism.ALL_EXPECTATIONS
                     )
+                    optimism.restoreOutput()
+                    optimism.restoreInput()
                 else:
                     established_expectations = None
             except Exception as e:
