@@ -38,6 +38,20 @@ from . import contexts
 from . import meta
 from . import snippets
 
+#---------#
+# Globals #
+#---------#
+
+CONFIG = None
+"""
+The configuration object that was passed to `setup`, or None if `setup`
+hasn't been called yet.
+"""
+
+
+#---------#
+# Helpers #
+#---------#
 
 def load_configuration(config_module_name):
     """
@@ -71,6 +85,68 @@ def load_configuration(config_module_name):
     return config
 
 
+def setup(config, specs_dir=None, templates_dir=None, resources_dir=None):
+    """
+    Performs common setup tasks. Requires a configuration object (see
+    `load_configuration`). Supplies defaults for specifications,
+    templates, and resources directories if they're not explicitly
+    provided.
+
+    This must be called before any of the launch_ functions are run.
+    """
+    global CONFIG
+    CONFIG = config
+    # Set specs directory for evaluate.py
+    if specs_dir is None:
+        specs_dir = os.path.join(config.BASE_DIR, "specs")
+
+    load.set_specs_dir(specs_dir)
+
+    # Set up reports system based on templates and resources directories
+    # from config
+    if templates_dir is None and config.TEMPLATES_DIRECTORY is not None:
+        templates_dir = os.path.join(
+            file_utils.potluck_src_dir(),
+            config.TEMPLATES_DIRECTORY
+        )
+
+    if resources_dir is None and config.RESOURCES_DIRECTORY is not None:
+        resources_dir = os.path.join(
+            file_utils.potluck_src_dir(),
+            config.RESOURCES_DIRECTORY
+        )
+
+    render.setup(templates_dir, resources_dir)
+
+
+def load_tasks_data(config):
+    """
+    Given a configuration object, loads the tasks.json tasks data file.
+    Logs a message about the file it loads data from.
+
+    Adds a "loaded_from" slot to the top-level dictionary it returns
+    that holds the filename from which the data was loaded.
+    """
+    # Load task meta-data
+    task_info_file = os.path.join(
+        config.BASE_DIR,
+        config.TASKS_FILENAME
+    )
+    logging.log(
+        f"Loading task metadata from '{task_info_file}'..."
+    )
+    with open(task_info_file, 'r') as fin:
+        result = json.load(fin)
+
+    result["loaded_from"] = task_info_file
+
+    return result
+
+
+#---------------------#
+# Core Task Functions #
+#---------------------#
+
 def generate_rubric(task_info, rubric_filename):
     """
     Generates a rubric for a single task, based on the task info and a
@@ -481,37 +557,9 @@ def evaluate_submission(
     return True
 
 
-def setup(config, specs_dir=None, templates_dir=None, resources_dir=None):
-    """
-    Performs common setup tasks. Requires a configuration object (see
-    `load_configuration`). Supplies defaults for specifications,
-    templates, and resources directories if they're not explicitly
-    provided.
-
-    This must be called before any of the launch_ functions are run.
-    """
-    # Set specs directory for evaluate.py
-    if specs_dir is None:
-        specs_dir = os.path.join(config.BASE_DIR, "specs")
-
-    load.set_specs_dir(specs_dir)
-
-    # Set up reports system based on templates and resources directories
-    # from config
-    if templates_dir is None and config.TEMPLATES_DIRECTORY is not None:
-        templates_dir = os.path.join(
-            file_utils.potluck_src_dir(),
-            config.TEMPLATES_DIRECTORY
-        )
-
-    if resources_dir is None and config.RESOURCES_DIRECTORY is not None:
-        resources_dir = os.path.join(
-            file_utils.potluck_src_dir(),
-            config.RESOURCES_DIRECTORY
-        )
-
-    render.setup(templates_dir, resources_dir)
-
+#--------------------#
+# Launcher Functions #
+#--------------------#
 
 def launch_job(
     job,
@@ -550,21 +598,12 @@ def launch_job(
     try:
         logging.log(f"This is potluck version {__version__}")
 
-        # Load task meta-data
-        task_info_file = os.path.join(
-            config.BASE_DIR,
-            config.TASKS_FILENAME
-        )
-        logging.log(
-            f"Loading task metadata from '{task_info_file}'..."
-        )
-        with open(task_info_file, 'r') as fin:
-            tasks_data = json.load(fin)
+        tasks_data = load_tasks_data(config)
 
         if taskid not in tasks_data["tasks"]:
             logging.log(
                 f"Fatal error: Task '{taskid}' does not exist in"
-                f" task info file '{task_info_file}'"
+                f" task info file '{tasks_data['loaded_from']}'"
             )
             exit(1)
 
