@@ -62,6 +62,85 @@ def len_as_text(html_string):
     return len(without_entities)
 
 
+def wrapped_fragments(line, width=80, indent=''):
+    """
+    Wraps a single line of text into one or more lines, yielding a
+    sequence of strings. Breaks on spaces/tabs only, and does not
+    attempt any kind of optimization but simply breaks greedily as close
+    to the given width as possible. The given indentation will be
+    prepended to each line, including the first.
+    """
+    # Apply indentation
+    line = indent + line
+
+    if len(line) <= width:
+        yield line
+        return
+
+    # Look backwards from width for a place we can break at
+    breakpoint = None
+    for i in range(width, len(indent), -1):
+        # (note: breaking on first char doesn't make sense)
+        c = line[i]
+        if c in ' \t':
+            breakpoint = i
+            break
+
+    # If we didn't find a breakpoint that would respect the
+    # target width, find the first one we can
+    if breakpoint is None:
+        for i in range(width, len(line)):
+            c = line[i]
+            if c in ' \t':
+                breakpoint = i
+
+    if breakpoint is None:
+        # Can't find any breakpoint :'(
+        yield line
+    else:
+        first = line[:breakpoint]
+        rest = line[breakpoint + 1:]
+        yield first
+        # Note: avoiding yield from because we want Python2 compatibility
+        for item in wrapped_fragments(rest, width, indent):
+            yield item
+
+
+def wrap_text_with_indentation(text, width=80):
+    """
+    Word-wraps the given text, respecting indentation, to the given
+    width. If there is a line that's more-indented than the given width,
+    wrapping will happen as aggressively as possible after that. Wraps
+    only at spaces. Replicates tab/space mix of indentation in the
+    output.
+
+    This function does NOT optimize for line lengths, and instead just
+    breaks each line greedily as close to the given width as possible.
+    """
+    lines = text.splitlines()
+    wrapped = []
+    for line in lines:
+        if len(line) <= width:
+            wrapped.append(line)
+        else:
+            indentation = ''
+            for c in line:
+                if c in ' \t':
+                    indentation += c
+                else:
+                    break
+
+            wrapped.extend(
+                wrapped_fragments(
+                    line[len(indentation):],
+                    width,
+                    indentation
+                )
+            )
+
+    return '\n'.join(wrapped)
+
+
 def create_help_button(help_content):
     """
     Returns an HTML string for a button that can be clicked to display
@@ -421,9 +500,9 @@ def html_traceback(exc=None, title=None, linkable=None):
 
             def replacer(match):
                 link = html_link_to_line(taskid, fn, int(match.group(1)))
-                return f'{fn}&quot;, line {link},'
+                return '{}&quot;, line {},'.format(fn, link)
 
-            pat = rf'{fn}&quot;, line ([0-9]+),'
+            pat = r'{}&quot;, line ([0-9]+),'.format(fn)
             result = re.sub(pat, replacer, result)
 
     pre = '<pre class="traceback">\n{}\n</pre>'.format(result)
@@ -480,14 +559,17 @@ def string_traceback(exc=None):
     return rewrite_traceback_filenames(raw, rewrites)
 
 
-def rewrite_traceback_filenames(raw_traceback, prefix_map):
+def rewrite_traceback_filenames(raw_traceback, prefix_map=None):
     """
     Accepts a traceback as a string, and returns a modified string where
     filenames have been altered by replacing the keys of the given
     prefix_map with their values. In addition, filenames which include a
     directory that ends in "__tmp" will have all directory entries up to
     and including that one stripped from their path.
+
+    If no prefix map is given, the value of REWRITES will be used.
     """
+    prefix_map = prefix_map or REWRITES
     result = raw_traceback
     for prefix in prefix_map:
         replace = prefix_map[prefix]
