@@ -33,16 +33,32 @@ def simplify(description):
     return '^^^' + stripped.casefold() + '$$$'
 
 
-def categorized_goal_path(goal_type, category, *descriptions):
+def all_row_trails(report_or_row, trail_prefix=None):
     """
-    Returns a goal path list suitable for use with `Expectation` which
-    starts by matching against a specific goal type grouping and then a
-    category grouping, as would be present in a rubric that used the
-    `potluck.rubrics.core_extras_categorized_metric`.
+    Visits each row & sub-row of a report (or a report row) one by one.
+    For each row visited, it yields a tuple containing that row, followed
+    by a trail: a list of the first description entry of each ancestor of
+    that row, starting from the top-level ancestor and going down and
+    including that row.
 
-    Note: this isn't actually that complicated...
+    If provided the given trail prefix (a list of strings) will be
+    included before the start of each trail.
     """
-    return [goal_type, category] + list(descriptions)
+    if trail_prefix is None:
+        trail_prefix = []
+
+    if "table" in report_or_row:
+        for entry in report_or_row["table"]:
+            yield from all_row_trails(entry, trail_prefix)
+    else:
+        desc = report_or_row["description"][0]
+        below = trail_prefix + [ desc ]
+        if "subtable" in report_or_row:
+            yield (report_or_row, below)
+            for entry in report_or_row["subtable"]:
+                yield from all_row_trails(entry, below)
+        else:
+            yield (report_or_row, below)
 
 
 class Expectation:
@@ -51,10 +67,13 @@ class Expectation:
     specific result within a report. These expectations can be tested
     to make sure that a specification is working as designed.
 
-    To specify which goal the expectation applies to, a list of strings
-    is provided; each must uniquely match against an item in a report
-    table at a specific level, with the next string matching against
-    that row's sub-table, and so on. These matches are performed in a
+    To specify which goal the expectation applies to, there are two
+    options: you can provide a fragment of the goal's identifier as a
+    string (it must match exactly one goal; see
+    `potluck.rubrics.Rubric.goals_by_id`), or you can provide a list of
+    strings, each must uniquely match against an item in a report table
+    at a specific level, with the next string matching against that row's
+    sub-table, and so on. These matches are performed in a
     case-insensitive manner with HTML tags stripped out, against the
     primary obfuscated description entry for each goal/category. The
     specified string only has to match part of the goal description, but
@@ -65,52 +84,70 @@ class Expectation:
     Because of these matching rules, for a rubric where the standard
     metric `potluck.rubrics.core_extras_categorized_metric` is used,
     goal paths are usually straightforward to construct when default
-    descriptions are in place. Some examples:
+    descriptions are in place. Some examples of both id-fragment and
+    goal-path methods:
 
     - For a core FunctionDef Check for function 'foo':
-       `[ "procedure", "core", "define foo" ]`
+        `"core.check:def-foo$"` OR
+        `[ "procedure", "core", "define foo" ]`
 
     - For an extra FunctionCall Check for 'bar' as a sub-rule of the
-      check above:
-       `[ "procedure", "extra", "define foo", "call bar" ]`
+        check above:
+        `"core.check:def-foo:call-bar$"` OR
+        `[ "procedure", "extra", "define foo", "call bar" ]`
 
-    - For a core trace test of function 'foo':
-       `[ "process", "core", "the foo function must" ]`
+    - For a core trace test of function 'foo', assuming it was created
+        with group_name "trace":
+        `"core.test:foo:trace"` OR
+        `[ "process", "core", "the foo function must" ]`
     - (note that one could also use:)
-       `[ "process", "core", "^the foo function must" ]`
+        `"^goal:core.test:foo:trace$"` OR
+        `[ "process", "core", "^the foo function must" ]`
 
-    - For a core result value test of function 'foo':
-       `[ "product", "core", "foo returns" ]`
+    - For a core result value test of function 'foo' (with no group_name):
+        `"core.test:foo$"` OR
+        `[ "product", "core", "foo returns" ]`
+        (Note for the ID version, the $ is important to distinguish from
+        the case above.)
 
-    - For a core printed output test of function 'foo':
-       `[ "behavior", "core", "foo prints" ]`
+    - For a core printed output test of function 'foo' (with group_name
+        "output"):
+        `"core.test:foo:output"` OR
+        `[ "behavior", "core", "foo prints" ]`
     """
-    def __init__(self, goal_path, expected_status):
+    def __init__(self, goal_spec, expected_status):
         """
-        The goal_path is a list of strings specifying how to find the
+        The goal_spec is a list of strings specifying how to find the
         goal in a report (strings are matched against descriptions to
-        find sub-tables). Finally, the expected evaluation result is
-        required, which should be one of the strings used for goal
-        statuses (see `potluck.rubrics.Goal`).
+        find sub-tables). Alternatively, the goal_spec may be a single
+        string, which will must match a single goal in the rubric using
+        the same rules as `potluck.rubrics.Rubric.goals_by_id`. The
+        expected evaluation result is also required, which should be one
+        of the strings used for goal statuses (see
+        `potluck.rubrics.Goal`).
 
-        Note that the precise goal_path an `Expectation` should have
+        Note that the precise goal_spec list an `Expectation` should have
         depends on the metric used and the details of how a
         `potluck.rubrics.Rubric` object formulates its overall report,
         because any top-level organizational report rows (e.g. for goal
-        types or categories) need to be accounted for.
+        types or categories) need to be accounted for. Specifying an
+        identifier fragment doesn't depend on the metric, but requires
+        understanding how identifiers are built up, and in some cases,
+        automatic deduplication of goal identifiers must be accounted
+        for.
 
-        For matching goals, the case-folded version of each goal_path
-        entry is checked using 'in' against a case-folded version of each
-        rubric entry at the relevant level. Exactly 1 rubric entry must
-        match.  The rubric entries also have HTML tags stripped out, and
-        have '^^^' added at the front and '$$$' at the end to aid in
-        disambiguation.
+        For matching using a goal spec that's a list of strings, the
+        case-folded version of each goal_spec entry is checked using 'in'
+        against a case-folded version of each rubric entry at the
+        relevant level. Exactly 1 rubric entry must match.  The rubric
+        entries also have HTML tags stripped out, and have '^^^' added at
+        the front and '$$$' at the end to aid in disambiguation.
 
         For example, if there are rubric entries named "Bug #1" and
         "Bug #11", an expectation for the "Bug #1" rubric entry could use
-        "bug #1$" as its goal_path entry.
+        "bug #1$" as its goal_spec entry.
         """
-        self.goal_path = goal_path
+        self.goal_spec = goal_spec
         self.expected_status = expected_status
 
     def check(self, report):
@@ -130,45 +167,103 @@ class Expectation:
         found = None
         trail = []
 
-        # Match at each level of our goal path
-        for match_key in self.goal_path:
-            # Match against descriptions at this level
-            matches_here = []
-            for row in rows_here:
-                match_against = simplify(row["description"][0])
-                look_for = match_key.casefold()
-                if look_for in match_against:
-                    matches_here.append(row)
+        if isinstance(self.goal_spec, str):
+            candidates = []
+            all_ids = []
+            for (row, trail) in all_row_trails(report):
+                if 'id' in row:
+                    all_ids.append(row['id'])
+                    if self.goal_spec in ('^^^' + row['id'] + '$$$'):
+                        candidates.append((row, trail))
 
-            # Check # of matching rows
-            if len(matches_here) != 1: # zero or multiple matches
-                if trail:
-                    where = "In " + ' → '.join(trail)
-                else:
-                    where = "At the top level of the report"
+            if (
+                len(all_ids) == 0
+            and (
+                    report["summary"]
+                 == "You did not submit any code for this task."
+                )
+            ):
+                return (
+                    False,
+                    "There was no submission.",
+                    None
+                )
 
+            if len(candidates) == 0:
                 options = '\n'.join(
-                    row['description'][0]
-                    for row in rows_here
+                    '#' + ident
+                    for ident in all_ids
                 )
                 return (
                     False,
                     (
-                        f"{where}, {len(matches_here)} goals matched"
-                        f" '{match_key}'. Goals here are:\n{options}"
+                        f"0 goals matched"
+                        f" '#{self.goal_spec}'. Available goal ids"
+                        f" are:\n{options}"
+                    ),
+                    None
+                )
+            elif len(candidates) > 1:
+                options = '\n'.join(
+                    '#' + row['id']
+                    for (row, trail) in candidates
+                )
+                return (
+                    False,
+                    (
+                        f"{len(candidates)} goals matched"
+                        f" '#{self.goal_spec}'. Matching goals"
+                        f" are:\n{options}"
                     ),
                     None
                 )
-            else: # a single match, as required
-                # Record the goal or other table row we found:
-                found = matches_here[0]
-                # Extend our trail
-                trail.append(found["description"][0])
-                # Enter next level of the table:
-                rows_here = found["subtable"]
 
-        # Strings for reporting our result
-        where = "In " + ' → '.join(trail)
+            # We found one match:
+            found, trail = candidates[0]
+
+            # String for reporting where we are
+            where = "In " + ' → '.join(trail)
+
+        else: # we assume it's a collection of strings
+            # Match at each level of our goal path
+            for match_key in self.goal_spec:
+                # Match against descriptions at this level
+                matches_here = []
+                for row in rows_here:
+                    match_against = simplify(row["description"][0])
+                    look_for = match_key.casefold()
+                    if look_for in match_against:
+                        matches_here.append(row)
+
+                # Check # of matching rows
+                if len(matches_here) != 1: # zero or multiple matches
+                    if trail:
+                        where = "In " + ' → '.join(trail)
+                    else:
+                        where = "At the top level of the report"
+
+                    options = '\n'.join(
+                        row['description'][0]
+                        for row in rows_here
+                    )
+                    return (
+                        False,
+                        (
+                            f"{where}, {len(matches_here)} goals matched"
+                            f" '{match_key}'. Goals here are:\n{options}"
+                        ),
+                        None
+                    )
+                else: # a single match, as required
+                    # Record the goal or other table row we found:
+                    found = matches_here[0]
+                    # Extend our trail
+                    trail.append(found["description"][0])
+                    # Enter next level of the table:
+                    rows_here = found["subtable"]
+
+            # Strings for reporting our result
+            where = "In " + ' → '.join(trail)
 
         # "found" should now be the matched goal's report row
         if found["status"] == self.expected_status:
@@ -213,11 +308,12 @@ def check_entire_report(
     for exp in all_expectations:
         success, expl, path = exp.check(report)
         if path is None:
+            if isinstance(exp.goal_spec, str):
+                gs = '#' + exp.goal_spec
+            else:
+                gs = ' → '.join(exp.goal_spec)
             raise ValueError(
-                "Unable to find expected goal:\n{}\n{}".format(
-                    " → ".join(exp.goal_path),
-                    expl
-                )
+                "Unable to find expected goal:\n{}\n{}".format(gs, expl)
             )
         c = coverage
         for entry in path:
@@ -318,7 +414,7 @@ def example(username):
         .setdefault(username, [])
 
 
-def expect(status, goal_type, category, *path):
+def expect(status, *id_or_path):
     """
     Creates an `Expectation` object and registers it under a given task
     ID and username. Arguments are:
@@ -327,18 +423,26 @@ def expect(status, goal_type, category, *path):
     - taskid: The task ID this expectation applies to.
     - username: The username whose submission the expectation applies
         to (usually a fake user set up for testing purposes).
-    - goal_type: The type of goal.
-    - category: The goal category.
     - path: One or more additional strings specifying which goal we're
-        targeting (see `Expectation`).
+        targeting (see `Expectation`). May also be a single string that
+        starts with '#' to specify the goal using its identifier instead
+        of a rubric-description-path. If it's a single string, it should
+        start with the goal type and then category when using the default
+        rubric metric.
     """
     mname = file_utils.get_spec_module_name()
+
+    if len(id_or_path) == 1 and id_or_path[0].startswith('#'):
+        goal_spec = id_or_path[0][1:]
+    else:
+        goal_spec = id_or_path
+
     EXPECTATIONS\
         .setdefault(mname, {})\
         .setdefault(CURRENT_EXAMPLE, [])\
         .append(
             Expectation(
-                categorized_goal_path(goal_type, category, *path),
+                goal_spec,
                 status
             )
         )
