@@ -90,6 +90,9 @@ from . import file_utils
 from . import html_tools
 from . import harness
 from . import render
+from . import rubrics
+from . import contexts
+from . import context_utils
 
 
 #---------#
@@ -393,7 +396,28 @@ class Snippet(specifications.TestGroup):
         caption = render.render_markdown(self.caption)
         post = render.render_markdown(self.postscript)
 
-        context_dicts = [ctx.create(base_context) for ctx in contexts]
+        context_dicts = []
+        for ctx in contexts:
+            try:
+                cdict = ctx.create(base_context)
+            except context_utils.ContextCreationError:
+                cdict = {
+                    # In case it's a Variables snippet
+                    "variable": "UNKNOWN",
+                    # for RunModule snippets
+                    "filename": "UNKNOWN",
+                    # for FunctionCalls snippets
+                    "function": "UNKNOWN",
+                    "args": (),
+                    "kwargs": {},
+                    # for Blocks snippets
+                    "block": "UNKNOWN",
+                    # For all kinds of snippets
+                    "value": "There was an error compiling this snippet.",
+                    "output": "There was an error compiling this snippet.",
+                    "error_log": html_tools.string_traceback()
+                }
+            context_dicts.append(cdict)
 
         # Strip out solution file paths from output, and perform final
         # edits if we have an editor function
@@ -407,7 +431,7 @@ class Snippet(specifications.TestGroup):
                     '&lt;soln&gt;'
                 )
 
-            if soln_path in context["error_log"]:
+            if "error_log" in context and soln_path in context["error_log"]:
                 context["error_log"] = context["error_log"].replace(
                     soln_path,
                     '&lt;soln&gt;'
@@ -842,6 +866,121 @@ class Blocks(Snippet):
         return result
 
 
+class Fakes(Snippet):
+    """
+    One or more fake snippets, which format code, printed output, stderr
+    output, and a result value (and possibly glammer additions) but each
+    of these things is simply specified ahead of time. Note that you want
+    to avoid using this if at all possible, because it re-creates the
+    problems that this module was trying to solve (namely, examples
+    becoming out-of-sync with the solution code). The specialization
+    methods of this kind of snippet are ignored and have no effect,
+    because no code is actually run.
+    """
+    def __init__(self, sid, title, caption, fake_contexts, postscript=''):
+        """
+        A snippet ID, title, and caption are required, along with a list
+        of fake context dictionary objects to be rendered as if they came
+        from a real test. Each dictionary must contain a "code" slot, and
+        may contain "value", "output", and/or "error_log" keys. If using
+        glammers, relevant keys should be added directly to these fake
+        contexts.
+
+        If "value" or "output" slots are missing, defaults of None and ''
+        will be added, since these slots are ultimately required.
+
+        A postscript is optional (see `Snippet`).
+        """
+        self.fake_contexts = fake_contexts
+        for ctx in self.fake_contexts:
+            if 'code' not in ctx:
+                raise ValueError(
+                    "Fake context dictionaries must contain a 'code'"
+                    " slot."
+                )
+            if 'value' not in ctx:
+                ctx['value'] = None
+            if 'output' not in ctx:
+                ctx['output'] = ''
+        super().__init__(sid, title, caption, [], postscript)
+
+    def create_goal(self):
+        """
+        We override TestGroup.create_goal to just create a dummy goal
+        which depends on dummy contexts.
+        """
+        ctx_list = []
+        for fake in self.fake_contexts:
+            def make_builder():
+                """
+                We need to capture 'fake' on each iteration of the loop,
+                which is why this extra layer of indirection is added.
+                """
+                nonlocal fake
+                return lambda _: fake
+            ctx_list.append(contexts.Context(builder=make_builder()))
+
+        return rubrics.NoteGoal(
+            self.taskid,
+            "fakeSnippetGoal:" + self.base_name,
+            (
+                "NoteGoal for fake snippet '{}'.".format(self.base_name),
+                "A fake NoteGoal."
+            ),
+            test_in={ "contexts": ctx_list }
+        )
+
+    def render_context(self, context):
+        """
+        Renders a context created for goal evaluation as HTML markup for
+        executing a block of code. Includes a Jupyter-style prompt for
+        input as well as the result value of the last line of the block
+        as long as that's not None.
+        """
+        src = context["code"]
+        value = context["value"]
+        captured = context.get("output", '')
+        captured_errs = context.get("error_log", '')
+
+        # Wrap faked inputs with spans so we can color them blue
+        captured = re.sub(
+            harness.FAKE_INPUT_PATTERN,
+            r'<span class="input">\1</span>',
+            captured
+        )
+
+        # Highlight the code block
+        blockrep = highlight_code(src)
+
+        result = (
+            '<span class="prompt input">In []:</span>'
+            '<div class="snippet-input">{}</div>\n'
+        ).format(blockrep)
+
+        if captured:
+            result += (
+                '<span class="prompt printed">Prints</span>'
+                '<div class="snippet-printed">{}</div>\n'
+            ).format(captured)
+
+        if captured_errs:
+            result += (
+                '<span class="prompt stderr">Logs</span>'
+                '<div class="snippet-stderr">{log}</div>\n'
+            ).format(log=captured_errs)
+
+        # Highlight the return value
+        if value is not None:
+            value = highlight_code(pprint_with_prefix(value, ""))
+
+            result += (
+                '<span class="prompt output">Out[]:</span>'
+                '<div class="snippet-output">{}</div>\n'
+            ).format(value)
+
+        return result
+
+
 #----------#
 # Glammers #
 #----------#
