@@ -8,4 +8,4 @@ import sys
 
 import pytest
 
-sys.exit(pytest.main(["--pyargs", "potluck.tests"]))
+sys.exit(pytest.main(["--pyargs", "potluck.tests"] + sys.argv[1:]))
